/*                              
     MAILLIST: A "stand-alone""list-server" like facility, with automatic
               storage of messages in an SRE-http message box.
             This is equivalent to the READMAIL facility of SRE-http.
             Among other advantages, it does not suffer from the GoServe
             induced premature timing out It can also be configured
             to run  at scheduled intervals.
             However, it does require some extra setup,

                        ------------------------
                           Introduction

Basic Requirements:

    Your server must have read/write access to an "inbox" directory of a
    mail server. For not-too-busy systems,  you can run SENDMAIL on
    the web server, and use it's MAIL directory.

        CAUTION: If you wish to use SENDMAIL to provide an "inbox", and
                 you also want to use ULTIMAIL to read "personal" mail
                 addressed to your server, you MUST set the OTHER_MAILBOX
                 and OTHER_ULTIMAIL parameters.

    For broadcast features to work, access to an SMTP Gateway if required
    (see the SMTP_GATEWAY variable below)


User configuation required:

  To use this, you will need to modify the values of  several (sets of) 
  variables (see below for further description).  Note that for a few
  parameters, a default will be used if the value is not set.

     INBOX_DIR -- the incoming mail storage directory
     FORUM.n   -- name of a "discussion group" (there may be 1..N different groups)
     USERLIST.n -- Allowed posters to the n=1..N discussion groups
     BROADCASTLIST.n  -- List of e-mail addresses to broadcast submissions to.
     AUTOSUBSCRIBE.n -- Allow SUBSCRIBE/UNSUBSCRIBE and set BROADCASTLIST=USERLIST
     AUTOVERIFY  -- Send a "verification" message to posters (default=YES).
     MAIL_ARCHIVE -- Where to archive messages after reading them from the inbox.
     MAIL_FREQ -- Freqency of mail processing (default=15 minutes)

     OUTFILE -- Output file; for status messages, error messages, etc.                    
     VERBOSE --  The amount of output information (0,1, or 2; 0 is the least).              
     SERVERNAME -- OPTIONAL. IP address of your server --the IP Domain to which mail
             should be sent.  Default is the "hostname" read from OS/2.
    MESSBOX_DIR -- Location of SRE-http "message box" .LOG files.                        
    SMTP_GATEWAY -- IP address of an SMTP Gateway mail-server.  Used when sending          
              verification mesages and when rebroadcasting messages.
              Default is the value of SERVERNAME (that is, the assumption is you
              are running SENDMAIL or some other SMTP server).
    GMTOFFSET -- Hours different from Greenwich mean time.
                 For example, Eastern Standard Time is -4.
                 If not included, GMT will not be used in replys.
    OTHER_MAILBOX -- If specifed, the directory to transfer files that are 
                     deposited in the SRE-http mailbox, but that do NOT 
                     match one of the FORUM.n entries. 
    OTHER_ULTIMAIL -- If files desposited into the OTHER_MAILBOX are
                      to be read by Ultimail, this instructs MAILLIST
                      to create the necessary .UML file.

 
                        -----------------------------------
                 Short description of user configured variables:

INBOX_DIR:     The directory your mail server(i.e.; SENDMAIL) uses to
               store incoming messages.

    The next several stem variables refer to n=1..N possible "discusson groups".

FORUM.n:   Required. A name (possibly with / in it).  These refer to
           "message boxes" (as used by SRE-http)-- .LOG files in the 
           MESSBOX_DIR directory.
                    DO NOT SPECIFY A FULLY QUALIFIED FILE NAME!

USERLIST.n:  Required. A filename.  If not fully qualified, then it is
             assumed to be relative to the MESSBOX_DIR.
             The file should contain a list of "allowed posters" for 
             "forum.n";  the list should
             contain E-Mail addresses (with possible wildcards).

BROADCASTLIST.n:  Optional. A file name. If not fully qualified, then it is
                  assumed to be relative to the MESSBOX_DIR.
                  Assuming the file exists, messages
                  will be rebroadcast to all e-mail addresses in this list.
                     Be careful you don't get into some kind
                      of infinite back-and-forth rebroadcast loop with
                      another server!

AUTOSUBSCRIBE.n:  Optional. A 0, 1, or 2.  1 means "allow automatic
                  subscription"; the USERLIST is used as the BROADCASTLIST,
                  and SUBSCRIBE and UNSUBSCRIBE "subjects" are used to
                  modify the USERLIST.  2 is the same, except the
                  poster is NOT sent the message.
                        A value of 0 (or if it is not specifed), suppresses
                        this.

       These next variables are global to all discussion groups.  Note that most
       of them are given default values if no value is set.

AUTOVERIFY:   0 or 1.  Send a verification note to the message sender.
              O for No, 1 for yes.
              Note: for automatic verification, you must have a valid entry
                    in the SRE-http SMTP_GATEWAY variable.
              The default is 1 (for YES).

MAIL_ARCHIVE:   The name of an archive directory to move read messages to.
                Set =0 if you want to delete messages after recording them.
                Note: the MAIL_ARCHIVE directory MUST be on the same
                disk as the INBOX_DIR (but not necessarily in the
                same directory tree).  The default is 0.


MAIL_FREQ:       Frequency of processing, in minutes.  That is,
                 MAILLIST will check the mail every MAIL_FREQ minutes (it
                 will "sleep" in the intervals between processing).
                 If you want MAILLIST to process mail once, then quit,
                 use a value of 0 (that is, set MAIL_FREQ=0).  The default 
                 is  15.

OUTFILE:         Name for output file (status messages, error messages ,etc.
                 results are written to it).  Note that these messages are
                 written in HTML format, so you should read this file with
                 your favorite web browser.  The default is MAILSTVU.HTM
                 in your MESSBOX_DIR directory.

SMTP_GATEWAY:    IP address of your SMTP mail gateway (used for AUTOVERIFY and
                 for broadcasting.  If not set, the value of SERVERNAME is used
                 (the assumption is that you are running SENDMAIL, or some other
                 SMTP server, on your machine).


VERBOSE:         Controls how much output is written and displayed.
                 0=minimal output, 1=moderate output, 2=lots of output.
                 Default=1.


SERVERNAME:      The IP name of your server -- it's the "domain" to which
                 mail should be sent.  The default is found by calling the
                 OS/2 "hostname" function.

MESSBOX_DIR:     The fully qualified directory in which SRE-http 
                 message boxes are stored. Also used as the default path 
                 for BROADCASTLIST and  USERLIST files.


OTHER_MAILBOX:   If specifed, the directory to transfer files that are
                 deposited in the, but that do NOT match one of the
                 FORUM.n entries. 

                 Use of OTHER_MAILBOX and OTHER_ULTIMAIL allows one to 
                 use ULTIMAIL as a mail reader with minimal hassle.
                 Note that if you wish to do this, be sure that:
                       1) INBOX_DIR is different then your ULTIMAIL
                          incoming directory.
                       2) SENDMAIL deposits mail in the INBOX_DIR, and
                          NOT in the ULTIMAIL incoming directory.
                 If not specified (or set equal to 0), the "none matching"
                 files will be left unchanged. 

                 For a discsussion of the use of SENDMAIL, ULTIMAIL, and
                 MAILLIST see MAILLIST.DOC

OTHER_ULTIMAIL:  If you "other mailbox" is to be used by ULTIMAIL, you should
                 set OTHER_ULTIMAIL=1.  This instructs MAILLIST to create
                 .UML files (which ULTIMAIL requires).


GMTOFFSET:        Hours different from GMT time.  For example,
                  U.S. Eastern Standard Time  is -5.  If not available,
                  GMT will not be used in reply messages.

                        -----------------------------------

                  Overriding defaults using request string options.

You can direct MAILLIST to process messages for a single discussion group
by using options when executing MAILLIST.
For example:
D:\GOSERVE>DODMAIL INBOX_DIR=dir_name FORUM=forun_2 USERLIST=usr_file AUTOVERIFY=0

In this example, ONLY messages sent to the forum_2 message-box
will be procesed (using usr_file, in the MESSBOX_DIR directory, as the userlist),
with no verification.


*/


/*  ------------------ Start of USER-configurable parameters section    ---- */
/* the values used below are just for illlustration purposes! */

/* INBOX_DIR: The directory where your mail server (eg; SENDMAIL) will
   place recieved e-mail.  */

INBOX_DIR = 'G:\GOSERV\MAIL'           /* enter an absolute directory */


/* List of "forums" to accept mail for. E-mail should be addressed
   to these forums. For example, if you have a INFO forum, then
   e-mail should be send to INFO@MYOFFICE.MYDAYJOB.ORG, and you should enter
   FORUM.1="INFO" .  You can enter as many FORUM.n entries as you want,
   just make sure that FORUM.0 contains the number of entries.
   Note that a .LOG file MUST exist (in the MESSBOX_DIR directory)
   for each FORUM.n entry. */
FORUM.0=2
FORUM.1='NEW1'
FORUM.2='NEW2'

/* For each forum, the filename containing a list of e-mail addresses
   of those allowed to submit messages (via e-mail) to this address.
   If not an absolute file, it is assumed to be relative to the
   MESSBOX_DIR directory */
USERLIST.1='NEW1.USR'
USERLIST.2='NEW2.USR'

/* Optional: for each forum, specify an absolute file name containing
   a "rebroadcast" list (the message will be sent to these other e-mail addresses,
   as well as saved in the message box. If you don't specify one, no
   rebroadcast will occur. This is ignored if AUTOSUBSCRIBE.n=1 or 2.
   If not an absolute file, it is assumed to be relative to the
   MESSBOX_DIR directory  */
BROADCASTLIST.1='NEW1.BRD'
BROADCASTLIST.2=' '                    

/* Optional: you can allow automatic subscription by specifying
  a non-0 value of AUTOSUBSCRIBE.n.  WIth AUTOSUBSCRIBE.n=1,
  the SUBSCRIBE and UNSUBSCRIBE "subjects" can be used to add/remove
  e-mail addresses from the userlist; and the userlist is used as the
  broadcast list (the BROADCASTLIST.n variable is ignored).
  AUTOSUBSCRIBE.n=2 is the same, except messages are not
  broadcast to the sender. 0 means "suppress this feature". */
autosubsribe.1=0
AUTOSUBSCRIBE.2=0


/* Send a "your message has been posted to the xxx discussion group"
   response to messages posters. */
AUTOVERIFY=0           /* 0= No,  1=Yes */

/* Where to archive messages after reading them.  Must be
   an absolute directory name, or set MAIL_ARCHIVE=0 if you
   want to delete messages (and NOT save in archive directory).
   NOTE: must be on the SAME disk as the INBOX_DIR (rename is used).
*/
MAIL_ARCHIVE = 'G:\goserv\mail\mailarch'           /* enter an absolute directory */


/* Frequency of activity.  Enter a "mail processing frequency",in minutes. 
After processing, MAILLIST will "sleep" for this many minutes, wake up, 
and look for new mail.
For example, MAIL_FREQ=15 means "process mail every 15 minute". */
MAIL_FREQ=1


/* Absolute file name: for output of results.  Note that results will
be in HTML format (it's best viewed with a web browser).*/
OUTFILE='G:\GOSERV\MESSAGE\MAILSTVU.HTM'

/* Quantity of output: 0=least, 1=medium , 2=lots of output */
VERBOSE=1               /* 0=minimal output, 1=more output, 2=lots of output, 3=too much  */

/*Servername: the name of the server you are running MAILLIST on.
  Used when sending messages.  If commented out, will use value of
  OS/2 hostname function. */
/*SERVERNAME=' '*/

/* Location of SRE-http message box files (XXX.LOG files). */
MESSBOX_DIR='G:\goserv\message'

/* Address of an SMTP gateway server (used to send verification notes and
   to rebroadcast message.  If commented out, will use value of SERVERNAME. */
/*SMTP_GATEWAY='' */


/* Number of hours difference from GMT (i.e.; us eastern standard = -5 ) */
gmtoffset=-5


/* Absolute directory to copy messages to non FORUM.n users; 0 means
"don't transfer it" */
other_mailbox='e:\tcpip\umail\server\inbox'

/* If this "other mailbox" is to be used by Ultimail, set other_ultimail=1 */
other_ultimail=1

/*  ------------------ End of USER-configurable parameters     ---- */

if \RxFuncQuery("SockLoadFuncs") then nop
else do
       call RxFuncAdd "SockLoadFuncs","rxSock","SockLoadFuncs"
       call SockLoadFuncs
end
/* Load up advanced REXX functions */
foo=rxfuncquery('sysloadfuncs')
if foo=1 then do
  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  call SysLoadFuncs
end
/* load up rexxlib functions */
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 call rxfuncadd 'rexxlibregister','rexxlib', 'rexxlibregister'
 call rexxlibregister
end



say '     MAILLIST: A "list-server" like facility, with automatic'
say '               storage of messages in a SRE-http message boxes. '
say ' (hit any key to continue ) '
foo=inkey('w')




seperator='!===============1x2y3z4a5b6c7d8e9f0g9h8i7j6k5l4m3n2o1===========!'
crlf='0d0a'x
errlist="" ; totsend=0
transaction=0
noarchive=0
if symbol('GMTOFFSET')<>'VAR' then gmtoffset=0
if datatype(gmtoffset)<>'NUM' then gmtoffset=0
gmtoffset=gmtoffset*3600

/* check if other_mailbox exists */
other_mailbox=strip(other_mailbox,'t','\')
if dosisdir(other_mailbox)=0 then other_mailbox=0


/* get all arguments from command line */
arglist=' '
do mm=1 to 25
   foo=arg(mm)
   if foo="" then leave
   arglist=arglist||' '||foo
end /* do */


do until arglist="" 
   parse var arglist a1 arglist
   parse upper var a1 avar '=' aval ;
   avar=upper(avar)
   aval=strip(upper(aval)) ; avar=strip(avar)
   if aval=' ' then iterate
  select
   when avar='INBOX_DIR' then do
        inbox_dir=aval
   end

   when avar="FORUM" then do
        forum.1=aval ; forum.0=1 ; 
   end  /* Do */

   when avar='USERLIST' then do   
       userlist.1=translate(aval,'\','/')
   end 
   when avar="BROADCASTLIST" then do
       broadcastlist.1=translate(aval,'\','/')
   end
   when avar="AUTOSUBSCRIBE" then do
       autosubscribe.1=aval
   end  /* Do */

   when avar='AUTOVERIFY' then do
       autoverify=aval
   end  /* Do */
   when avar="MAIL_ARCHIVE" then do
        mail_archive=translate(aval,'\','/')
   end
   when avar="MAIL_FREQ" then do
       mail_freq=aval
   end  /* Do */

   when avar='MESSBOX_DIR' then do
        messbox_dir=aval
   end
   when avar="SMTP_GATEWAY" then do 
       smtp_gateway=aval
   end /* do */
   when avar="VERBOSE" then do
      verbose=aval
   end /* do */
   when avar="SERVERNAME" then do
         servername=aval
   end /* do */
   when avar="OUTFILE" then do
      outfile=aval
   end
   otherwise
       say " Unknown argument: " avar "," aval
 end
end /* getting command line options */

/* cleanup some general stuff */

inbox_dir=translate(inbox_dir,'\','/')
inbox_dir=strip(strip(inbox_dir),'t','\')||'\'

if symbol('MAIL_FREQ')<>'VAR' then mail_freq=15
if datatype(mail_freq)<>'NUM' then
   mail_freq=15

if symbol('VERBOSE')<>'VAR' then verbose=1
if datatype(verbose)<>"NUM" then verbose=1

if symbol('MAIL_ARCHIVE')<>var then mail_archive=0
if mail_archive<>0 then do
    if filespec('D',mail_archive)<>filespec('D',inbox_dir) then do
        foo=bad_archive(mail_archive,inbox_dir)
        signal gotobed
    end
    mail_archive=strip(strip(mail_archive),'t','\')||'\'
end
if symbol('SERVERNAME')<>'VAR' then servername=get_hostname()
if symbol('SMTP_GATEWAY')<>'VAR' then smtp_gateway=servername

messbox_dir=strip(strip(messbox_dir),'t','\')||'\'

if symbol('AUTOVERIFY')<>'VAR' then autoverify=1
if datatype(AUTOVERIFY)<>'NUM' then autoverify=1

if symbol('OUTFILE')<>'VAR' then outfile=messbox_dir||'MAILSTVU.HTM'

/* write header info */
 call lineout OUTFILE, "<html><head><title>MAILLIST status</title>"
 call lineout OUTFILE, "</head><body>"
 call lineout outfile,' '
 call lineout outfile,' <hr> <h2> MAIL-LIST Status Report </h2>'

say " Writing status and error messages to " outfile
say " This server's address: " servername
say " The SMTP (e-mail) Gateway is: " SMTP_GATEWAY
say " Mail inbox directory = " inbox_dir
say " Message box directory = " messbox_dir
if other_Mailbox>0 then do
  say " Transferring non FORUM mail to " other_mailbox
  if other_ultimail=1 then
      say "   ... and creating .UML files. "
end
do ijl=1 to forum.0 by 8
  tko=' '
  do emm=ijl to min(forum.0,ijl+7)
     tko=tko||' 'forum.emm
  end
  say " Monitoring messages to: " tko 
end   


/********* JUMP HERE TO PROCESS MAIL AGAIN (AFTER WAKING UP)   ******/
monsters:  nop

/* check status of userlists and broadcastlists */
do mm=1 to forum.0

   ndone.mm=0

/* check .n stuff. NOTE: userlist.mm MUST exist */
  if symbol('FORUM.'||mm)<>'VAR' then do     /* check forum.n */
       iterate                  /* bad numbering, just ignore */
  end  /* Do */

  hforum=strip(translate(forum.mm,'\','/'))
  aff=strip(hforum,,'\')
  aff2=messbox_dir||aff||'.log'
  if stream(aff2,'c','query exists')=' ' then do  /* does message box exist */
      anerr=" Warning: MailList -- missing message box file, will not accept mail for  " forum.mm
      errlist=errlist||'<li>'||anerr
      if verbose>1 then say anerr
      forum.mm=' '
      iterate
  end
  forum.mm=aff

/* userlist.mm MUST exist */
  if symbol('USERLIST.'||mm)<>'VAR' then do     /* check userlist.n */
      anerr=" Warning: MailList -- missing userlist, will not accept mail for  " forum.mm
      errlist=errlist||'<li>'||anerr
      if verbose>1 then say anerr
      forum.mm=' '
      iterate
  end

/* check user list */
  atmp=strip(translate(userlist.mm,'\','/'))
  if abbrev(atmp,'\')=1 | filespec('D',atmp)<>' ' then /* absolute */
          userlist.mm=atmp
  else
          userlist.mm=messbox_dir||atmp
  if stream(userlist.mm,'c','query exists')=' ' then do
        anerr=" Warning: MailList -- bad userlist, will not accept mail for  " forum.mm
        errlist=errlist'<li>'anerr
       if verbose>1 then say anerr
        forum.mm=' '
        iterate
  end
 
  if symbol('BROADCASTLIST.'||mm)<>'VAR' | broadcastlist.mm=0   ,
                                        | broadcastlist.mm=' ' then do
        broadcastlist.mm=' '
  end
  else do
     atmp=strip(translate(broadcastlist.mm,'\','/'))
     if abbrev(atmp,'\')=1 | filespec('D',atmp)<>' ' then /* absolute */
          broadcastlist.mm=atmp
     else
          broadcastlist.mm=messbox_dir||atmp
     if stream(broadcastlist.mm,'c','query exists')=' ' then do
            anerr="Warning: MailList -- missing broadcastlist for FORUM= " forum.mm
            if verbose>1 then say anerr
            errlist=errlist'<li>'anerr
            broadcastlist.mm=' '
     end
  end

   if symbol('AUTOSUBSCRIBE.'||mm)<>'VAR' | autosubscribe.mm=' ' ,
                                      | autosubscribe.mm=0   then do  /* check for autosubscribe*/
         autosubscribe.mm=0
   end
   else do
        if wordpos(autosubscribe.mm,'0 1 2')=0 then do
              say " Bad value for AUTOSUBSCRIBE " autosubscribe.mm
              autosubscribe.mm=0
        end
        else do
            if autosubscribe.mm >0 then do
               if broadcastlist.mm<>' ' then do
                   anerr='Warning: MailList -- Autosubscribe specified, broadcast list ignored:' broadcastlist.mm
                   if verbose>1  then say anerr
                  errlist=errlist'<li>'anerr
               end  /* Do */
               broadcastlist.mm=userlist.mm
           end          /* autosubscribe>0 */
        end  /* valid autosubscribe*/
   end


end /* do */

if verbose>2 & errlist<>' ' then
  say " Warnings: " errlist


/* get a list of all files in inbox_dir */

allf=sysfiletree(inbox_dir||'*.*',fils,'F')
if allf<>0 | if fils.0 = 0 then do
   foo=byebye(inbox_dir)
   signal gotobed
end

/* scan each file in fils.
1) Look for a  'To: xxx@aaa.bbb.ccc' field -- before first empty line.
  If none, skip this file
2) If found such a field, check that the xxx matches one of the FORUM.n
   entries.  If not, skip this file (possibly move it to OTHER_MAILBOX)
3) If matches a FORUM.n, look for a  From: poo bear <yyy@ddd.eee.fff>
   field.  Look in USERLIST.n (same n as FORUM.n) for <yyy@ddd.eee.fff>
   (or wildcard matches).  If not found, then skip file.
4) If here, extract from the file:
     The user name (from the From: field)
     E-mail address (same as used in 3)
     Subject
     Date
     Message (everything after first blank line, including attachments)

5) Write the items in 4 to the FORUM.n message box (in the MESSBOX_DIR directory).

6) If autoverify, send a "we got your message" response to the e-mail
   address 
7) Rebroadcast if desired.
8) Get next file
*/

ndone.0=0

if verbose>0 then say fils.0 " possible messages to be examined."

/* extract the header from the first message in the inbox */
do afil=1 to fils.0


  fils.afil=word(fils.afil,words(fils.afil))

   yow=fileread(fils.afil,filelines,,'E')
   if yow <= 0 then iterate
   endhead=0
   do rr=1 to filelines.0
      if filelines.rr=' ' then do
          endhead=rr
          leave
      end
      headlines.rr=strip(upper(filelines.rr))
   end /* do */
   if endhead=0 then iterate            /*no contents, skip */

  if verbose>2 then say " Examining: " fils.afil

 toforum=' ';fromwho=' '; asubject=' '; adate=' '

/* 1) check for various fields (To:, From:, Subject:, Date:) */
  do rr=1 to endhead
      aline=headlines.rr
      select
        when abbrev(aline,'TO:')=1 then do
           aline=substr(aline,4)

           toforum=word(aline,words(aline))     /* get the last word */
           toforum=strip(translate(toforum,' ','<>'))
        end
        when abbrev(aline,'FROM:')=1 then
          fromwho=strip(substr(strip(filelines.rr),6))
        when abbrev(aline,'SUBJECT:')=1 then
          asubject=strip(substr(strip(filelines.rr),9))
        when  abbrev(aline,'DATE')=1 then
          adate=strip(substr(strip(filelines.rr),6))
        when abbrev(aline,'X-NOARCHIVE:')=1 then
           noarchive=1
        otherwise
              nop
       end              /* select */

  end

  if toforum=' '  then iterate         /* no To: field, */

   parse var toforum aforum '@' forum_host ; aforum=strip(aforum)
   if aforum=' ' then iterate           /* no forum specified -- skip */


if verbose> 2 then say " Examining mail file:  to " aforum ", from " fromwho
  
/*2) see if aforum matches a forum.n entry */
   nthforum=0
   do whichf=1 to forum.0
       if aforum<>upper(forum.whichf) then iterate
       nthforum=whichf ; leave
   end /* do */
   
   if nthforum=0 then do 
       if other_mailbox<>0 then do
           useit=other_mailbox||'\'||filespec('n',fils.afil)
           foo=doscopy(fils.afil,useit,'r')
           if verbose>2 then say "Moving message to: " useit
           if foo<>0 then say  foo ": errror transferring " fils.afil
           if foo=0 then foo=sysfiledelete(fils.afil)
           if other_ultimail=1 then do
                ali=useit||' '||upper(aforum)
                mm1=lastpos('.',useit)
                if mm1=0 then
                   tf=useit||'.UML'
                else
                   tf=delstr(useit,mm1)||'.UML'
                foo=charout(tf,ali,1)
                if foo>0 & verbose>1 then say " MAILLIST: Problem creating .UML file"
                if verbose>2 then say " Created: " useit ' , ' tf
           end

       end  /* Do */
   end

   if userlist.nthforum=' ' then iterate /* no or bad user list */

/* extract personal name and e-mail from the From: field */
   theadd0=word(fromwho,words(fromwho))
   theadd0=strip(theadd0,'l','<'); theadd0=strip(theadd0,'t','>')

   theadd0=strip(theadd0)
   if words(fromwho)=1 then
      parse var theadd0 thename '@' .
   else
       parse var fromwho  thename .
   theadd=upper(theadd0)
   aa=fileread(userlist.nthforum,users,,'E')

   if aa<0 | users.0=0 then iterate       /* no such userlist, skip this file */


/* is this a "subscribe" or "unsubscribe" message? */
   if  autosubscribe.nthforum>0 then do
      if upper(strip(asubject))='SUBSCRIBE' then do
           foo=sref_open_read(userlist.nthforum,30,'BOTH')
           if foo<0 then do
              anerr= '  Warning: UserList is busy, try again later: ' userlist.nthforum
              errlist=errlist'<li>'anerr
              if verbose>1 then say anerr
              foo=remove_message(fils.afil)
              iterate
           end
           dingo=chars(userlist.nthforum)
           foob=charin(userlist.nthforum,chars(userlist.nthforum),1)
           if foob<>'1a'x then dingo=dingo+1
           poo=charout(userlist.nthforum,theadd0||crlf,dingo)
           foo=stream(userlist.nthforum,'c','close')
           amessage=' Your subscription to the '||aforum||' discussion group '
           amessage=amessage||crlf||'at '||forum_host||' has been accepted.'||crlf
           call send_verify('Subscribe to '||aforum)
           if verbose>1 then
              say  theadd0 " Subscribed to " aforum
           errlist=errlist'<li> 'theadd0' subscribed to ' aforum||crlf

           foo=remove_message(fils.afil)
           iterate
      end

      if strip(upper(asubject))='UNSUBSCRIBE' then do
            foo=remove_user(theadd0,userlist.nthforum)
            select
               when  foo>0 then do
                 amessage=' Your subscription to the '||aforum||' discussion group '
                 amessage=amessage||crlf||' has been canceled.'||crlf
                 call send_verify('UnSubscribe from '||aforum)
                 errlist=errlist'<li> 'theadd0' unsubscribed from ' aforum||crlf
              end
              when foo=0 then do
                 amessage=' You attempted to unsubscribe from a discussion group, ('||aforum||')'
                 amessage=amessage||crlf||' to which you were NOT subscribed.'||crlf
                 call send_verify('UnSubscribe from '||aforum)
              end  /* Do */
              otherwise
                 nop            /* try later */
            end
            if verbose>0 then
              say  theadd0 " unSubscribed to " aforum
            foo=remove_message(fils.afil)

            iterate
      end
   end  /*  AUTOSUBSCRIBE        */

/* 3) scan userlist file for match, possibly wildcard, to theadd */
   okay_user=check_user_list(theadd)
   if okay_user<0 then do
        errlist=errlist'<li> Message from unallowed poster: 'theadd0
   end  /* Do */
   if okay_user<1 then iterate

/* 4 ) Already got subject, date, and from.  But some conversions necessary?
Note: write out as:
!===============1x2y3z4a5b6c7d8e9f0g9h8i7j6k5l4m3n2o1===========!
Message-id:  19961008200859.80.16@151.121.65.143
Date: 20:08:59   8 Oct 1996
From:   mom
E-mail:  foo@bar.net
Subject:  no drugs

Contents:
ths stuff in the file

*/

/* create unque message id */
   p1=date('s')
   p2=time('n') ; p2=delstr(p2,3,1); p2=delstr(p2,5,1)
   p3=dostid()||"_"||dospid()
   transaction=transaction+1 ; p4=transaction
   buildme=CRLF||seperator||crlf||'Message-id: '||AFIL||'_'||p1||p2||'.'||p3||'.'||p4||'@'||p5||crlf

/* transform e-mail date  (smtp standard is: Sat, 19 Oct 1996 11:06:14 -0400)
   to message box format: 11:06:14  19 Oct 1996
*/
    parse var adate foo ','  d1 d2 d3 at .
    date2=at||' '||d1||' '||d2||' '||d3

    buildme=buildme||'Date: '||date2||crlf
    buildme=buildme||'From: '||thename||crlf
    buildme=buildme||'E-mail: '||theadd0||crlf
    buildme=buildme||'Subject: '||asubject||crlf||crlf||'Contents: '||crlf

/* now add all lines in the remainder of the message */
    buildme2=''
    do oo=endhead+1 to filelines.0
       buildme2=buildme2||filelines.oo||crlf
    end
    buildme=buildme||buildme2||crlf

/* get names in the broadcast list */
    namesl=make_brdlist(broadcastlist.nthforum,autosubscribe.nthforum,theadd0)


  if noarchive==0 then do
/* and the namd of the "message box" to store it in */
     lfile=messbox_dir||aforum||'.log'  /* default message box */

     foo=sref_open_read(lfile,30,'BOTH')
     if foo<0 then do
        anerr= "  Warning: MailList -- Unable to find message box: " lfile
       errlist=errlist'<li>'anerr
       if verbose>1 then say anerr
       iterate
     end
     dingo=chars(lfile)
     foob=charin(lfile,dingo,1)
     if foob<>'1a'x then dingo=dingo+1
     foo=charout(lfile,buildme,dingo)
     foo=stream(lfile,'c','close')
     if verbose>1 then say " Recording message: " asubject "; to  " aforum " message box "
     ndone.nthforum=ndone.nthforum+1
  end

  ndone.0=ndone.0+1


/* delete or move the NOW-RECORDED message file */
  foo=remove_message(fils.afil)

  broadcast_name=toforum

/* 6) Send verification to theadd0? */
   if  autoverify=1 & smtp_gateway<>' 'then do
       amessage='Your e-mail submission: ' asubject ||crlf
       amessage=amessage||'to the ' || aforum ||' discussion group at '||servername||crlf
       amessage=amessage||' ... was successfully processed on '||date2
       call send_verify('Status of submission ')
    end


/* 7) Rebroadcast (use nameSL determined above) ? */
 
   if namesl=' ' then  iterate          /* no or empty broadcast list */


/* put together a forwarding messabe */
   bsubject='Subject: '||asubject||' (forwarded)'

   dacontents=date3||crlf

      dacontents='Mime-Version: 1.0'||crlf||'Content-Type: text/plain; charset="us-ascii"'||crlf
      dacontents=dacontents||'To:  '||Aforum||' discussion group <>'||crlf
      dacontents=dacontents||'From: Message processor <'||toforum||'>'||crlf

      dacontents=dacontents||bsubject||crlf
 
/* put notes in message body */
      dacontents=dacontents||crlf||'Message sent to '||aforum ' discussion group at '||forum_host||crlf
      dacontents=dacontents||'Date recieved: '||date2||crlf
      dacontents=dacontents||'Sent by: '||fromwho||crlf
      dacontents=dacontents||'Subject: '||asubject||crlf
      dacontents=dacontents||crlf||buildme2
   if verbose>1 then
       say ' Attempting to broadcast message to ' words(namesl) ' in list: ' broadcastlist.nthforum


/* Broadcast 50 names at a time -- too many names can overwhelm some SMTP gateways
(such as SENDMAIL) */
   oof="" ; wct=0
   do iww=1 to words(namesl) 
     oof=oof||' '||word(namesl,iww)
     wct=wct+1
     if wct=50 then do
        if verbose>1 then
           say " Sending mail to " wct " recipients "
        status=sref_mailit(namesl,dacontents,smtp_gateway ,'<'||broadcast_name||'>',verbose)
        if status=0 then do
          anerr=" Warning: MailList -- Problem encountered when broadcasting! "
          if verbose>1  then say anerr 
          errlist=errlist'<li>'anerr   
        end
        wct=0 ; oof=""  ; totsend=totsend+words(namesl)
     end        /* wct=50 */
   end          /* iww loop */
   
   if oof<>' ' then do          /* broadcast remainder of (or possibly entire short) list */
        hyy=words(oof)
        if verbose>1 then
           say " Sending mail to " hyy " recipients "
        status=sref_mailit(namesl,dacontents,smtp_gateway ,'<'||broadcast_name||'>',verbose)
        if status=0 then do
          anerr=" Warning: MailList -- Problem encountered when broadcasting! "
          if verbose>1  then say anerr 
          errlist=errlist'<li>'anerr   
        end

        totsend=totsend+words(namesl)
   end
        

end             /* scan all files llop */


/* write status report */
if ndone.0 =0 & errlist='' then do
  say " No messages (from a listed address) were found. "
  signal gotobed
end

say Ndone.0 " incoming messages were processed. " totsend " messages were mailed "
  call write_newsection(0)
  call lineout OUTFILE,' <h2> ' ndone.0 ' messages processed by MAILLIST  </h2>'
 if ndone.0=0 then do
        call lineout OUTFILE,' No mail messages were found for currently defined discussion groups '
 end
 else do
    call lineout OUTFILE,' The following message boxes were updated: '
    call lineout OUTFILE,'  <ol>'
    do jt=1 to forum.0
         if ndone.jt=0 then iterate
         call lineout OUTFILE,' <li> ' ndone.jt ' messages written to ' forum.jt
    end
    call lineout OUTFILE,'  </ol>'
 end  
 if totsend>0 then
    call lineout outfile, totsend " messages were mailed "


 if errlist<>' ' then do
      call lineout OUTFILE,' <p> Notes: <ul>'
      call lineout OUTFILE,errlist
      call lineout OUTFILE,'</ul><p>'
 end  /* Do */
/* all done, write final end-of-html elements */
 call lineout OUTFILE


/* ---- here after mail has been processed */
gotobed:
if mail_freq=0 then do
    say " Bye.. "
    exit 
end /* do */

say " It is now " time('n') ' on ' date('n')
say ' MAILLIST will "wake up" in ' mail_freq  " minutes."
foo=syssleep(mail_freq*60)
errlist='' ; totsend=0
signal monsters


/* -----------  write a " header" for a mail checking  -- */
write_newsection:

call lineout outfile, '<p> <hr>'
nowd=date('n') 
nowt=time('n')
call lineout outfile,'<h3> ' nowt  nowd ' </h3> '
call lineout outfile,'<br>'
return 0




/* ------------------  check user list  ---------- */
/* check user list for theadd, return 1 if found;  0 if not found or if
a "not this guy" found. "not this guy" is VERY IMPORTANT if you have
wildcards (otherwise, you risk endless recursion whenever an error
message is returned from your smtp gateway!
*/
check_user_list:procedure expose users.
parse arg theadd
   isok=0
   parse var theadd dname '@' ddomain
   ddomain2=translate(ddomain,' ','.')

   do   aad=1 to users.0
      a1=strip(translate(users.aad,' ','000d0a1a09'x))
 
      if abbrev(strip(a1),';')=1 | a1=' ' then iterate

      isnot=1
      if abbrev(a1,'NOT')=1 then do
          isnot=-1
          a1=strip(substr(a1,4))
      end

      if theadd=a1 then       /* exact match */
           return isnot         /* if isnot=-1, DO NOT LET IN */
      
      if a1='*' then do      /*  generic wildcard */
          if isnot=-1 then iterate   /* NOT * is ignored */
          isok=1                /* if no NOT match further down, then accept */
      end  /* Do */

/* try non-generic wildcard */
      parse upper var a1  nname '@' ndomain
      if nname<>'*' & nname<>dname   then  iterate    /* does name  NOT match */

/* see if domain matches */
      ndomain2=translate(ndomain,' ','.')
      if words(ddomain2) <> words(ndomain2) then iterate /* no matching domain */
      do mw=1 to words(ddomain2) 
         if word(ddomain2,mw)=word(ndomain2,mw) | word(ndomain2,mw)='*' then
                iterate
         iterate aad            /* no match */
      end

/* if here, got a matching domain  */
     if isnot=-1 then return isnot    /* forbiddens trump * permissions */

     isok=1             /* but keep looking for possible NOT entries */

   end                     /*aad: search users */
   return isok




/* ************************* */
/* remoove (Delete or archive) a message file */
remove_message:procedure expose mail_archive errlist verbose
parse arg thefil
  IF MAIL_ARCHIVE=0 then do
      FOO=SYSFILEDELETE(thefil)          /* DELETE THE MESSAGE */
      if foo<>0 then do
         anerr=' Warning: MAILLIST -- error ' foo ' trying to delete ' thefil
         if verbose>1 then say anerr
         errlist=errlist'<li>'anerr
      end
  end
  ELSE do                       /* MOTE TO ARCHIVE DIRECTORY */
      eeh=mail_archive||filespec('N',thefil)
      FOO=DOSRENAME(theFIL,eeh)
      if foo=0 then do
         anerr=' Warning: MAILLIST -- could not archive ' thefil ' to ' eeh ' (delete instead)'
         FOO=SYSFILEDELETE(thefil)          /* DELETE THE MESSAGE */
         if verbose>1 then say anerr
         errlist=errlist'<li>'anerr
      end
  end
  return 1



/* ************************ */
/* get list e-mail addresses to broadcast to */
make_brdlist:procedure expose verbose smtp_gateway errlist
parse arg ylfile,asub,theadd0

 if ylfile=' ' | smtp_gateway=' ' then return ' '

/* else, scarf up the  broadcast_list */
   if stream(ylfile,'c','query exists')=' ' then do
           anerr="MailList -- Broadcast denied (no message list found: " ylfile
           if verbose>1 then say anerr
           errlist=errlist'<li>'anerr
           return ' '
   end
   puds=fileread(ylfile,yllist)

   namesl=' '
   do ik=1 to yllist.0               /* grab the names */
         yllist.ik=translate(yllist.ik,' ','000d0a1a09'x)
         if abbrev(strip(yllist.ik),';')=1 | yllist.ik=' ' then iterate  /*comment skips */
         if pos('*',yllist.ik)>0 then do
             if verbose>1  then do
                  anerr='Warning: MailList -- Bad entry in broadcast list not used: ' yllist.ik
                  errlist=errlist'<li>'anerr
                  if verbose > 1 then say anerr
                  iterate
             end  /* Do */
          end

/* if autosubscribe=2, then check for "own name", and do not include if
   found */
        if yllist.ik=theadd0 & asub=2 then iterate
         namesl=namesl||' '||yllist.ik
   end

  if namesl=" " then do
           anerr='Warning: MailList -- Empty broadcast list '
           if verbose>1 then say anerr
           errlist=errlist'<li>'anerr
           iterate      /* since no more actions for this file */
  end  /* Do */
  return namesl




/*************/
byebye:procedure expose OUTFILE
parse arg inbox_dir

    call lineout OUTFILE, '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
    call lineout OUTFILE, "<html><head><title>MAILLIST failed</title>"
    call lineout OUTFILE, "</head><body>"
    call lineout OUTFILE,' <h2>Error in MAILLIST </h2>'
    call lineout OUTFILE,' Error reading IN-box directory: ' inbox_dir

/* all done, write final end-of-html elements */
    call lineout OUTFILE
    say "MAIL LIST Failed -- no In Box Directory "
    exit


/*************/
bad_archive:procedure expose OUTFILE
parse arg mail_archive,inbox_dir

    call lineout OUTFILE, '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
    call lineout OUTFILE, "<html><head><title>MAILLIST failed</title>"
    call lineout OUTFILE, "</head><body>"
    call lineout OUTFILE,' <h2>Error in MAILLIST </h2>'
    call lineout OUTFILE,' Bad mail archive directory: ' maiL_archive
    call lineout OUTFILE,' <p>(should be in same drive as In-box directory: ' inbox_dir ')'

/* all done, write final end-of-html elements */
    call lineout OUTFILE

    say " MAIL LIST failed -- Bad archive directory "
    exit


/*  ----------------------------------------------------- */
send_verify:            /* send verify message. uses globals */
parse arg dasub
      gmtoff=gmtoffset
      gmtoff=100*gmtoff/(60*60)
      if gmtoff=0 then gmtoff=' '
      date3='Date: ' left(date('w'),3)||', '||date('n')||' '||time('n')||' '||gmtoff
      dacontents=date3||crlf

      dacontents='Mime-Version: 1.0'||crlf||'Content-Type: text/plain; charset="us-ascii"'||crlf
      dacontents=dacontents||'To: '||thename||'<'||theadd0||'>'||crlf
      dacontents=dacontents||'From: '||aforum||' Discussion group <_@_>'||crlf

      dacontents=dacontents||'Subject:'||dasub||crlf

      dacontents=dacontents||crlf||amessage

      if verbose>1 then
        say " Attempting to mail verification to "  theadd0
      status=sref_mailit(theadd0,dacontents,smtp_gateway ,'<>',verbose)

      if status=0 then do
          anerr=" Warning: MailList -- unable to send verification to: " theadd0
          if verbose>1 then say anerr
          errlist=errlist'<li>'anerr
      end

      totsend=totsend+1

      return 0


/* -------------------------------------- */
remove_user:procedure expose errlist verbose
parse upper arg theadd0,ufile

crlf = '0d0a'x
aneol=crlf

ause=sref_open_read(ufile,30,'BOTH')
if ause<0 then                  /* couldn't get it */
  return -1
ause=charin(ufile,1,chars(ufile))
ause=translate(ause,' ','001a'x)
filelines.0=0
i=0 ; nfound=0
do until ause=""
  parse var ause aline (aneol) ause
  aline=strip(aline)
  if upper(aline)=upper(theadd0) then do
      nfound=nfound+1
      iterate
  end  /* Do */
  i=i+1
  filelines.i=aline
end
filelines.0=i
a=stream(ufile,'c','close')
foo=filewrite(ufile,filelines,'R',i,1)
return nfound


/* ----------------------------------------------------------------------- */
/* OPEN_READ: keep trying to open a file (for msec seconds
. Argumentes:
        afile == file to open
        msec == quit trying after msec seconds
        howopen = open mode (READ WRITE BOTH READ ) -- default is READ
  Returns
    Status, with values
        -1 = no such file
        -2 = error opening (say, NEW specified but file exists), or timeout
        >0 = seconds it took to open
*/
/* ----------------------------------------------------------------------- */

sref_open_read:procedure
parse upper arg afile , msec , howopen .
debug=0

howopen=translate(howopen)

if afile=0 | afile="" then do
   if debug=1 then   say "OPEN_READ: No file name provided "
   return -1             /*no such file flat */
end

/* DISALLOW wildcarded files (they cause trouble below */
if pos('*',afile)>0 | pos('?',afile)>0 then do
    if debug=1 then  say "OPEN_READ: No wildcards allowed "
    return -1
end

isfile=stream(afile,'c','query exists') ;
if howopen="NEW"  then do
    if isfile="" then
        isfile=afile
    else do
        if debug=1 then  say "OPEN_READ: NEW file already exists "
       return -1
    end  /* Do */
end
else do
   if isfile=""  then do
       if debug=1 then  say 'OPEN_READ: Could not find ' afile
      return -1             /*no such file */
   end 
end

sec1=time('RESET')
foy=time('ELAPSED')

do until time('ELAPSED') > msec
    select
    when howopen='BOTH' then
       inuse=stream(isfile,'c','open')
    when howopen='WRITE'| howopen="NEW" then do
         inuse=stream(isfile,'c','open write')
    end
    otherwise do
        inuse=stream(isfile,'c','open read')
    end
    end
    if inuse<>'READY:' then do
        foo=syssleep(1)                  /* wait a second, and try again */
        iterate
      end
/* Else, it's openable */
    gog=time('ELAPSED')
    return gog+0.01
end
 if debug=1 then  say " OPEN_READ: no time "
return -2                /* could not open in alloted time */


/* This is MAILIT: an SRE-http procedure */
/* ============================================================================ 
MAILIT: A  REXX procedure to mail a note to several recipients.
.
.  Called as:  status=mailit(address_list,message_string,smtp_gateway,optional_sender_address)
.
. address_list: A space delimited list of addresses
. message_string: A message string, that may contain CRLFs (and may be rather long)
. smtp_gateway: The address of your SMTP server 
. optional_senders_address: The sender's address (optional)
.        If sender's address is not given, it will be generated using the servername() function.
.
. Example:
. status=mailit("BOB@HIS.NET" , "This is my message ", "MAIL.MY.ORG" )
.    or
. status= mailit("BOB@HIS.NET JILL@HER.COM " , "This is my message ", ,
.             "MAIL.MY.ORG", "HARRY@MY.ORG")
.
.NOTES:
. SMTP gateway addresses.
.   If you are using an INTERNET provider, use the POP or SMTP gateway address.
.
.   If no such address is available, you can try running the OS/2 SENDMAIL program, and
.   use your hostname as the SMTP gateway address.
.   For example, assuming that SENDMAIL.UML is in c:\mptn\etc,
.   before starting GoServe you should   start sendmail with:
.       [C:]START SENDMAIL -bd -q30m -Cc:\mptn\etc\sendmail.uml
.   Alternatively, you can use AUTOSTART sendmail using the TCPCFG program.
.
. Note on recipient addresses:
.    The message will be sent to each address.  Since the addresses are
.    sent and then the message, this allows for fairly rapid mailing to multiple
.    individuals.
.
. Acknowledgment:
.     This is a modification of a routine written by Dave Briccetti (see acknowledgment at
.     bottom of file)
.     It was modified by Daniel Hellerstein for use in SRE-http. 4/96
.
.
============================================================================ */

sref_mailit:procedure
parse arg whotos , themessage  , smtpaddress , sender_address,verbose

smtpaddress=strip(smtpaddress)
sender_address=strip(sender_address)

/* set some constants */
CRLF                    = '0d0a'x
TRUE                    = 1
FALSE                   = 0
REPLYTYPE_OK            = '2'   /* SMTP reply code first byte */
REPLY_START_MAIL_INPUT  = '354' /* SMTP reply code */

if sender_address="" then do
   sendinghost=servername()
   sendinguser="WebServer"
   sender_address=sendinguser||'@'||sendinghost
end
 
MailServer  = smtpaddress  /* Mail server, leave blank if you are connected through a LAN */


if EstablishProtocol() = FALSE then
    return 0

/* The protocol initiated, we'll now send the message to each
   recipient */

call SendMsgToEachRecipient

/* QUIT ends the protocol */
    
CmdReply = TransactSmtpCommand(socket, 'QUIT', 1)

/* Close the socket */
    
call SockSoClose socket

return 1


/* ========================================================================= */
SetConstants:
/* ========================================================================= */

return


/* ========================================================================= */
EstablishProtocol:
/* ========================================================================= */

socket = ConnectToMailServer(MailServer)    
if socket <= 0 then 
do
    if verbose>1 then 
          say 'Could not connect to mail server'
    return FALSE
end

CmdReply = GetCmdReply(socket)

if left(CmdReply, 1) \= REPLYTYPE_OK then 
do
    if verbose>1 then 
          say 'Could not establish protocol'
    return FALSE
end

/* Send the extended hello, in case this SMTP server supports
   SMTP extensions */
        
CmdReply = TransactSmtpCommand(socket, 'EHLO', 1)
        
if left(CmdReply, 1) = REPLYTYPE_OK then 
do
    /* That worked, so enable extended SMTP processing.  If
       the response to the EHLO indicates support for SIZE,
       enable our use of that feature */
              
    SmtpExtensionsSupported = TRUE
    if pos('250 SIZE', CmdReply) > 0 | pos('250-SIZE', CmdReply) > 0 then
        SizeExtensionSupported = 1
end
else
do
    /* The server didn't recognize the EHLO so we'll go with
       the regular HELO */
              
    SmtpExtensionsSupported = FALSE
    SizeExtensionSupported  = FALSE
    CmdReply = TransactSmtpCommand(socket, 'HELO', 1)
end

if left(CmdReply, 1) = REPLYTYPE_OK then 
    return TRUE
else
    return FALSE


/* ========================================================================= */
SendMsgBody: 
/* ========================================================================= */

/* DATA tells the server that the body of the message is coming.  It
   should reply with a code meaning "go ahead." */

CmdReply = TransactSmtpCommand(socket, 'DATA', 1)
if substr(CmdReply, 1, 3) = REPLY_START_MAIL_INPUT then 
do
    /* Send the data, followed by a '.' on a line by itself to 
       indicate the end of the message */
    CmdReply = TransactSmtpCommand(socket, MsgFileContents || CRLF || '.', 0)
end

return


/* ========================================================================= */
SendMsgToEachRecipient:
/* ========================================================================= */

/*
MsgFileContents = charin(MessageFile, 1, chars(MessageFile))
call stream MessageFile, 'c', 'close'
*/

msgfilecontents=themessage
MsgFileContents = strip(MsgFileContents, 't', '1a'x) 
  /* Strip EOF */

        
/* MAIL FROM identifies the sender.  The SIZE= extension
   provides the size of the message, to allow the 
   server to quickly refuse messages bigger than it wants. */
           
MailFromCmd = 'MAIL FROM:' ||sender_address

if SizeExtensionSupported then
    MailFromCmd = MailFromCmd 'SIZE=' || length(MsgFileContents)
    
CmdReply = TransactSmtpCommand(socket, MailFromCmd, 1)



if left(CmdReply, 1) = REPLYTYPE_OK then
  whotos=translate(whotos,' ','000d0a1a09'x)
  do while whotos<> ""
     parse var whotos whoto whotos

     RecipientEmailAddress = strip(whoto)
     if RecipientEmailAddress \= '' then    /* RCPT identifies the intended recipient of the message */
         CmdReply = TransactSmtpCommand(socket,,
                         "RCPT TO:" || RecipientEmailAddress, 1)
/*say " cmdreply " cmdreply*/
  end /* do */

call SendMsgBody

return


/* ========================================================================= */
ConnectToMailServer: procedure
/* ========================================================================= */

parse arg MailServer
socket = 0

/* Open a socket to the mail server.  (The Sock* functions are
   documented in the REXX Socket book in the Information folder
   in the OS/2 System folder */

call SockInit
if SockGetHostByName(MailServer, 'host.!') = 0 then do
    if verbose>1 then 
         say 'Could not get host by name' errno h_errno
end
else do
    socket = SockSocket('AF_INET','SOCK_STREAM',0)
    address.!family = 'AF_INET'
    address.!port = 25          /* the standard SMTP port */
    address.!addr = host.!addr
    if SockConnect(socket, 'address.!') = -1 then  do
      if verbose>1 then
            say 'Could not connect socket' errno h_errno
    end
end
return socket


/* ========================================================================= */
GetCmdReply: procedure
/* ========================================================================= */

parse arg socket

CRLF = '0d0a'x

/* Receive the response to the SMTP command into a variable.  Use
   more than one socket read if necessary to collect the whole 
   response. */
   
if SockRecv(socket, 'CmdReply', 200) < 0 then do
   say 'Error reading from socket' errno h_errno
   exit 0
end

ReadCount = 1
MaxParts = 10

do while ReadCount < MaxParts & right(CmdReply, 2) \= CRLF  
    if SockRecv(socket, 'CmdReplyExtra', 200) < 0 then do
        say 'Error reading from socket'
    end
    CmdReply = CmdReply || CmdReplyExtra
    ReadCount = ReadCount + 1
end

return CmdReply


/* ========================================================================= */
TransactSmtpCommand: 
/* ========================================================================= */

parse arg socket, Cmd, SayCmd

/* Send a command to the SMTP server, echoing it to the display
   if requested */

if SayCmd then 
    if verbose>1 then 
           say " Command is "  Cmd
rc = SockSend(socket, Cmd || CRLF)
return GetCmdReply(socket)


/* ------------- Acknowledgement  ------------------------------
Most of this e-mail routine was obtained from:

Send Email Message to a Mailing List Via SMTP, Using the REXX Socket Interface
Written by a novice REXX programmer
     Dave Briccetti, November 1995
     daveb@davebsoft.com, http://www.davebsoft.com

  May be used for any purpose
  Thanks to REXX expert <a href=http://www.quercus-sys.com>Charles Daney</a>
  and internet expert David Singer for looking over the code
----------------------------------------------------------    */



/* get the hostname (aa.bb.cc) for this machine */
get_hostname: procedure
    do queued(); pull .; end                   /* flush */
    address cmd '@hostname'  '| rxqueue'    
    parse pull hostname                        
    return hostname

