/*
     READMAIL: A "list-server" like SRE-http facility, with automatic
               storage of messages in an SRE-http message box.

                        ------------------------
                           Introduction

Basic Requirements:

    Your server must have read/write access to an "inbox" directory of a
    mail server. For not-too-busy systems,  you can run SENDMAIL on
    the web server, and use it's MAIL directory.

    For broadcast features to work. requires access to an SMTP Gateway (as
    set in the SMTP_GATEWAY SRE-http variable).
  
    Only clients with a SUPERUSER privilege are allowed to invoke this 
    procedure.


User configuation required:

  To use this, you will need to modify the values of  several (sets of) 
  variables (see below for further description):

     INBOX_DIR -- the incoming mail storage directory. If not specified,
                  the value of the MAILBOX_DIR SRE-http variable is used.
     FORUM.n   -- name of a "discussion group" (there may be 1..N different groups)
     USERLIST.n -- Allowed posters to the discussion group
     BROADCASTLIST.n  -- List of e-mail addresses to broadcast submissions to.
     AUTOSUBSCRIBE.n -- Allow SUBSCRIBE/UNSUBSCRIBE and set BROADCASTLIST=USERLIST
     AUTOVERIFY  -- Send a "verification" message to posters. Default value is NO.
     MAIL_ARCHIVE -- Where to archive messages after reading them from the inbox.
                     If not specified, messages are deleted after being processed.


                        -----------------------------------
                 Short description of user configured variables:

INBOX_DIR:     The directory your mail server(i.e.; SENDMAIL) uses to
               store incoming messages.  If not set, the value of the 
               SRE-http MAILBOX_DIR variable is used.

    The next several stem variables refer to n=1..N possible "discusson groups".

FORUM.n:   Required. A name (possibly with / in it).  These refer to
           "message boxes" (as used by SRE-http)-- .LOG files in the 
           MESSBOX_DIR directory.
                    DO NOT SPECIFY A FULLY QUALIFIED FILE NAME!

USERLIST.n:  Required. A filename.  If not fully qualified, then it is
             assumed to be relative to the MESSBOX_DIR.
             The file should contain a list of "allowed posters" for 
             "forum.n";  the list should
             contain E-Mail addresses (with possible wildcards).

BROADCASTLIST.n:  Optional. A file name. If not fully qualified, then it is
                  assumed to be relative to the MESSBOX_DIR.
                  Assuming the file exists, messages
                  will be rebroadcast to all e-mail addresses in this list.
                      Be careful you don't get into some kind
                      of infinite back-and-forth rebroadcast loop with
                      another server!

AUTOSUBSCRIBE.n:  Optional. A 0, 1, or 2.  1 means "allow automatic
                  subscription"; the USERLIST is used as the BROADCASTLIST,
                  and SUBSCRIBE and UNSUBSCRIBE "subjects" are used to
                  modify the USERLIST.  2 is the same, except the
                  poster is NOT sent the message.
                        A value of 0 (or if it is not specifed), suppresses
                        this.

       These next two variables are global to all discussion groups.

AUTOVERIFY:   0 or 1.  Send a verification note to the message sender.
              O for No, 1 for yes.
              Note: for automatic verification, you must have a valid entry
                    in the SRE-http SMTP_GATEWAY variable.


MAIL_ARCHIVE:   The name of an archive directory to move read messages to.
                Set =0 if you want to delete messages after recording them.
                Note: the MAIL_ARCHIVE directory MUST be on the same
                disk as the INBOX_DIR (but not necessarily in the
                same directory tree).



                        -----------------------------------

                  Overriding defaults using request string options.

You can direct READMAIL to process messages for a single discussion group
by using options when invoking READMAIL.
For example:
  READMAIL?INBOX_DIR=dir_name&FORUM=forun_2&USERLIST=user_list_file&AUTOVERIFY=0

In this example, ONLY messages sent to the forum_2 message-box
will be procesed (using usr_file as the userlist), with no verification.
and only from individuals listed in user_list_file)



                        -----------------------------------
                             Other READMAIL files.

  For more details, see READMAIL.DOC

  For an example of how using request string options, see READMAIL.HTM.

*/


/*  ------------------ Start of USER-configurable parameters     ---- */

/* Insert the directory where your mail server (eg; SENDMAIL) will
   place recieved e-mail.  If set to 0, the MAIL subdirectory
   of the "ETC" directory (as set by the ETC OS/2 environment variable)
   is used (i.e.; D:\MPTN\ETC\MAIL). */

INBOX_DIR = 'G:\GOSERV\MAIL'           /* enter an absolute directory */


/* List of "forums" to accept mail for. E-mail should be addressed
   to these forums. For example, if you have a INFO forum, then
   e-mail should be send to INFO@FOO.BAR.NET, and you should enter 
   FORUM.1="INFO" .  You can enter as many FORUM.n entries as you want,
   just make sure that FORUM.0 contains the number of entries */
FORUM.0=3
FORUM.1='LIST1'
FORUM.2='DINOS'
forum.3='easy'

/* For each forum, the filename containing a list of e-mail addresses
  of those allowed to submit messages (via e-mail) to this address.
   If not an absolute file, it is assumed to be relative to the
   MESSBOX_DIR directory */
USERLIST.1='LIST1.USR'
USERLIST.2='DINOS.USR'

/* Optional: for each forum, specify an absolute file name containing
   a "rebroadcast" list (the message will be sent to these other e-mail addresses,
   as well as saved in the message box. If you don't specify one, no
   rebroadcast will occur. This is ignored if AUTOSUBSCRIBE.n=1 or 2
   If not an absolute file, it is assumed to be relative to the
   MESSBOX_DIR directory  */
BROADCASTLIST.1='LIST1.BRD'
broadcastlist.2=' '                     /* ignored, given autosubscribe.2=2 */

/* Optional: you can allow automatic subscription by specifying
  a non-0 value of AUTOSUBSCRIBE.n.  WIth AUTOSUBSCRIBE.n=1,
  the SUBSCRIBE and UNSUBSCRIBE "subjects" can be used to add/remove
  e-mail addresses from the userlist; and the userlist is used as the
  broadcast list (the BROADCASTLIST.n variable is ignored).
  AUTOSUBSCRIBE.n=2 is the same, except the mail is not
  broadcast to the sender. 0 means "suppress this feature". */
autosubsribe.1=0
AUTOSUBSCRIBE.2=1


/* Send a "your message has been posted to the xxx discussion group"
   reponse. */
AUTOVERIFY=0           /* 0= No,  1=Yes */

/* Where to archive messages after reading them.  Must be
   an absolute directory name, or set MAIL_ARCHIVE=0 if you
   want to delete messages (and NOT save in archive directory).
   NOTE: must be on the SAME disk as the INBOX_DIR (rename is used).
*/
MAIL_ARCHIVE = 'G:\goserv\mail\mailarch'           /* enter an absolute directory */

/*
Time limit parameter:
   To avoid time-outs whilst processing a message, you can set the
   TIME_REQUIRED parameter.  If less than proportion of time
   remains before GoServe kills the connection, ReadMail will
   not start processing the next message.  In such cases, you'ld
   need to resubmit a request to READMAIL (to handle messages
   that had not been processed yet).
   Note that this offers only partial protection against unwanted
   time-outs (with possible incomplete message broadcasts).  To be
   safe, set this to a large value (less than 1.0), and set the GoServe
    "end client  after inactive" to be a large value.
   Note: lower values means "use more time to process messages" 
   (smaller safety margin).
*/

TIME_REQUIRED=.2             /* A value between 0.0 and 1.0 */



/*  ------------------ End of USER-configurable parameters     ---- */

seperator='!===============1x2y3z4a5b6c7d8e9f0g9h8i7j6k5l4m3n2o1===========!'
crlf='0d0a'x
errlist=""
noarchive=0

parse arg  ddir, tempfile, reqstrg,list,verb ,uri,user, ,
          basedir ,workdir,privset,enmadd,transaction,verbose, ,
         servername,host_nickname,homedir

if verb="" then do
   say " This SRE-http add-on is NOT meant to be run from the command line."
   say " You might want to try MAILLIST.CMD"
   exit
end  /* Do */



list=translate(list, ' ', '+'||'090a0d'x)  /* Whitespace, etc. */


/* Allow request string options to override defaults (but
only does one message box) */

list=packur(list)
do until list=""
   parse upper var list a1 '&' list
   parse var a1 avar '=' aval ; 
   aval=strip(packur(aval))

   if avar='INBOX_DIR' then do
        inbox_dir=aval
   end

   if avar="FORUM" then do
        forum.1=aval ; forum.0=1 ; 
   end  /* Do */

   if avar='USERLIST' then do   
       userlist.1=translate(aval,'\','/')
   end 
   if avar="BROADCASTLIST" then do
       broadcastlist.1=translate(aval,'\','/')
   end
   if avar="AUTOSUBSCRIBE" then do
       autosubscribe.1=aval
   end  /* Do */

   if avar='AUTOVERIFY' then do
       autoverify=aval
   end  /* Do */
   if avar="MAIL_ARCHIVE" then do
        mail_archive=translate(aval,'\','/')
   end
   if avar="TIME_REQUIRED" then do
       time_required=aval
   end  /* Do */
end /* do */

/* cleanup some general stuff */
if inbox_dir=' ' then inbox_dir=value(enmadd||'MAILBOX_DIR',,'os2environment')
inbox_dir=translate(inbox_dir,'\','/')
inbox_dir=strip(strip(inbox_dir),'t','\')||'\'

if datatype(time_required)<>'NUM' then
   time_required=0.2

if time_required<0 then time_required=0.1
if time_required>1.0 then time_required=0.9

limit_secs=extract('limittimeinactive')*(1-time_required)

isnow=time('r')


if mail_archive<>0 then do
    if filespec('D',mail_archive)<>filespec('D',inbox_dir) then do
        foo=bad_archive(mail_archive,inbox_dir)
        return 'FILE ERASE TYPE text/html NAME' tempfile
    end
    mail_archive=strip(strip(mail_archive),'t','\')||'\'
end

messbox_dir=value(enmadd||'MESSBOX_DIR',,'os2environment')
messbox_dir=strip(strip(messbox_dir),'t','\')||'\'

smtp_gateway=value(enmadd||'SMTP_GATEWAY',,'os2environment')


/* check status of userlists and broadcastlists */
do mm=1 to forum.0

   ndone.mm=0

/* check .n stuff. NOTE: userlist.mm MUST exist */
  if symbol('FORUM.'||mm)<>'VAR' then do     /* check forum.n */
       iterate                  /* bad numbering, just ignore */
  end  /* Do */

  hforum=strip(translate(forum.mm,'\','/'))
  aff=strip(hforum,,'\')
  aff2=messbox_dir||aff||'.log'
  if stream(aff2,'c','query exists')=' ' then do  /* does message box exist */
      anerr=" Warning: ReadMail -- missing message box file, will not accept mail for  " forum.mm
      errlist=errlist||'<li>'||anerr
      if verbose>1 then say anerr
      forum.mm=' '
      iterate
  end
  forum.mm=aff

/* userlist.mm MUST exist */
  if symbol('USERLIST.'||mm)<>'VAR' then do     /* check userlist.n */
      anerr=" Warning: ReadMail -- missing userlist, will not accept mail for  " forum.mm
      errlist=errlist||'<li>'||anerr
      if verbose>1 then say anerr
      forum.mm=' '
      iterate
  end

/* check user list */
  atmp=strip(translate(userlist.mm,'\','/'))
  if abbrev(atmp,'\')=1 | filespec('D',atmp)<>' ' then /* absolute */
          userlist.mm=atmp
  else
          userlist.mm=messbox_dir||atmp
  if stream(userlist.mm,'c','query exists')=' ' then do
        anerr=" Warning: ReadMail -- bad userlist, will not accept mail for  " forum.mm
        errlist=errlist'<li>'anerr
       if verbose>1 then say anerr
        forum.mm=' '
        iterate
  end
 
  if symbol('BROADCASTLIST.'||mm)<>'VAR' | broadcastlist.mm=0   ,
                                        | broadcastlist.mm=' ' then do
        broadcastlist.mm=' '
  end
  else do
     atmp=strip(translate(broadcastlist.mm,'\','/'))
     if abbrev(atmp,'\')=1 | filespec('D',atmp)<>' ' then /* absolute */
          broadcastlist.mm=atmp
     else
          broadcastlist.mm=messbox_dir||atmp
     if stream(broadcastlist.mm,'c','query exists')=' ' then do
            anerr="Warning: ReadMail -- missing broadcastlist for FORUM= " forum.mm
            if verbose>1 then say anerr
            errlist=errlist'<li>'anerr
            broadcastlist.mm=' '
     end
  end

   if symbol('AUTOSUBSCRIBE.'||mm)<>'VAR' | autosubscribe.mm=' ' ,
                                      | autosubscribe.mm=0   then do  /* check for autosubscribe*/
         autosubscribe.mm=0
   end
   else do
        if wordpos(autosubscribe.mm,'0 1 2')=0 then do
              say " Bad value for AUTOSUBSCRIBE " autosubscribe.mm
              autosubscribe.mm=0
        end
        else do
            if autosubscribe.mm >0 then do
               if broadcastlist.mm<>' ' then do
                   anerr='Warning: ReadMail -- Autosubscribe specified, broadcast list ignored:' broadcastlist.mm
                   if verbose>1  then say anerr
                  errlist=errlist'<li>'anerr
               end  /* Do */
               broadcastlist.mm=userlist.mm
           end          /* autosubscribe>0 */
        end  /* valid autosubscribe*/
   end


end /* do */

/* get a list of all files in inbox_dir */

allf=sysfiletree(inbox_dir||'*.*',fils,'F')
if allf<>0 | if fils.0 = 0 then do
   foo=byebye(inbox_dir)
   fii=value('SREF_PREFIX',,'os2environment')
   if fii<>'' then 
        foo=sref_gos('FILE ERASE TYPE text/html NAME' tempfile)
   else
      'FILE ERASE TYPE text/html NAME' tempfile
end

/* scan each file in fils.
1) Look for a  'To: xxx@aaa.bbb.ccc' field -- before first empty line.
  If none, skip this file
2) If found such a field, check that the xxx matches one of the FORUM.n
   entries.  If not, skip this file.
3) If matches a FORUM.n, look for a  From: poo bear <yyy@ddd.eee.fff>
   field.  Look in USERLIST.n (same n as FORUM.n) for <yyy@ddd.eee.fff>
   (or wildcard matches).  If not found, then skip file.
4) If here, extract from the file:
     The user name (from the From: field)
     E-mail address (same as used in 3)
     Subject
     Date
     Message (everything after first blank line, including attachments)

5) Write the items in 4 to the FORUM.n message box (in the MESSBOX_DIR directory).

6) If autoverify, send a "we got your message" response to the e-mail
   address 
7) Rebroadcast if desired.
8) Get next file
*/

ndone.0=0

/* extract the header from the first message in the inbox */
do afil=1 to fils.0

/* are we out of time? */
  foo=time('e')
  if foo>limit_secs then do
      errlist=errlist'<li><p> Ran out of time: please resubmit to process remaining messages</b>'
      leave
  end  /* Do */


  fils.afil=word(fils.afil,words(fils.afil))

   yow=fileread(fils.afil,filelines,,'E')
   if yow <= 0 then iterate
   endhead=0
   do rr=1 to filelines.0
      if filelines.rr=' ' then do
          endhead=rr
          leave
      end
      headlines.rr=strip(upper(filelines.rr))
   end /* do */
   if endhead=0 then iterate            /*no contents, skip */

 toforum=' ';fromwho=' '; asubject=' '; adate=' '

/* 1) check for various fields (To:, From:, Subject:, Date:) */
  do rr=1 to endhead
      aline=headlines.rr
      select
        when abbrev(aline,'TO:')=1 then do
           aline=substr(aline,4)
           toforum=word(aline,words(aline))     /* get the last word */
           toforum=strip(translate(toforum,' ','<>'))
        end
        when abbrev(aline,'FROM:')=1 then
          fromwho=strip(substr(strip(filelines.rr),6))
        when abbrev(aline,'SUBJECT:')=1 then
          asubject=strip(substr(strip(filelines.rr),9))
        when  abbrev(aline,'DATE:')=1 then
          adate=strip(substr(strip(filelines.rr),6))
        when abbrev(aline,'X-NOARCHIVE:')=1 then
           noarchive=1
        otherwise
              nop
       end              /* select */

  end



  if toforum=' '  then iterate         /* no To: field, */


   parse var toforum aforum '@' forum_host
   if aforum=' ' then iterate           /* no forum specified -- skip */

  
/*2) see if aforum matches a forum.n entry */
   nthforum=0
   do whichf=1 to forum.0
       if aforum<>upper(forum.whichf) then iterate
       nthforum=whichf ; leave
   end /* do */
   if nthforum=0 then iterate           /* no such forum */
   if userlist.nthforum=' ' then iterate  /* no or bad user list */

/* extract personal name and e-mail from the From: field */
   theadd0=word(fromwho,words(fromwho))
   theadd0=strip(theadd0,'l','<'); theadd0=strip(theadd0,'t','>')

   theadd0=strip(theadd0)
   if words(fromwho)=1 then
      parse var theadd0 thename '@' .
   else
       parse var fromwho  thename .
   theadd=upper(theadd0)

   aa=fileread(userlist.nthforum,users,,'E')
   if aa<0 | users.0=0 then iterate       /* no such userlist, skip this file */

/* is this a "subscribe" or "unsubscribe" message? */
   if  autosubscribe.nthforum>0 then do
      if upper(strip(asubject))='SUBSCRIBE' then do
           foo=sref_open_read(userlist.nthforum,30,'BOTH')
           if foo<0 then do
              anerr= '  Warning: UserList is busy, try again later: ' userlist.nthforum
              errlist=errlist'<li>'anerr
              if verbose>1 then say anerr
              foo=remove_message(fils.afil)
              iterate
           end
           dingo=chars(userlist.nthforum)
           foob=charin(userlist.nthforum,chars(userlist.nthforum),1)
           if foob<>'1a'x then dingo=dingo+1
           poo=charout(userlist.nthforum,theadd0||crlf,dingo)
           foo=stream(userlist.nthforum,'c','close')
           amessage=' Your subscription to the '||aforum||' discussion group '
           amessage=amessage||crlf||'at '||forum_host||' has been accepted.'||crlf
           call send_verify('Subscribe to '||aforum)
           if verbose>1 then
              say  theadd0 " Subscribed to " aforum
           errlist=errlist'<li> 'theadd0' subscribed to ' aforum||crlf

           foo=remove_message(fils.afil)
           iterate
      end

      if strip(upper(asubject))='UNSUBSCRIBE' then do
            foo=remove_user(theadd0,userlist.nthforum)
            select
               when  foo>0 then do
                 amessage=' Your subscription to the '||aforum||' discussion group '
                 amessage=amessage||crlf||' has been canceled.'||crlf
                 call send_verify('UnSubscribe from '||aforum)
                 errlist=errlist'<li> 'theadd0' unsubscribed from ' aforum||crlf
              end
              when foo=0 then do
                 amessage=' You attempted to unsubscribe from a discussion group, ('||aforum||')'
                 amessage=amessage||crlf||' to which you were NOT subscribed.'||crlf
                 call send_verify('UnSubscribe from '||aforum)
              end  /* Do */
              otherwise
                 nop            /* try later */
            end
            if verbose=1 then
              say  theadd0 " unSubscribed to " aforum
            foo=remove_message(fils.afil)

            iterate
      end
   end  /*  AUTOSUBSCRIBE        */

/* 3) scan userlist file for match, possibly wildcard, to theadd */
   okay_user=check_user_list(theadd)
   if okay_user<0 then do
        errlist=errlist'<li> Message from unallowed poster: 'theadd0
   end  /* Do */
   if okay_user<1 then iterate

/* 4 ) Already got subject, date, and from.  But some conversions necessary?
Note: write out as:
!===============1x2y3z4a5b6c7d8e9f0g9h8i7j6k5l4m3n2o1===========!
Message-id:  19961008200859.80.16@151.121.65.143
Date: 20:08:59   8 Oct 1996
From:   mom
E-mail:  foo@bar.net
Subject:  no drugs

Contents:
ths stuff in the file

*/

/* create unque message id */
   p1=date('s')
   p2=time('n') ; p2=delstr(p2,3,1); p2=delstr(p2,5,1)
   p3=extract('serverport')
   p4=transaction
   p5=extract('serveraddr')
   buildme=CRLF||seperator||crlf||'Message-id: '||AFIL||'_'||p1||p2||'.'||p3||'.'||p4||'@'||p5||crlf

/* transform e-mail date  (smtp standard is: Sat, 19 Oct 1996 11:06:14 -0400)
   to message box format: 11:06:14  19 Oct 1996
*/
    parse var adate foo ','  d1 d2 d3 at .
    date2=at||' '||d1||' '||d2||' '||d3

    buildme=buildme||'Date: '||date2||crlf
    buildme=buildme||'From: '||thename||crlf
    buildme=buildme||'E-mail: '||theadd0||crlf
    buildme=buildme||'Subject: '||asubject||crlf||crlf||'Contents: '||crlf

/* now add all lines in the remainder of the message */
    buildme2=''
    do oo=endhead+1 to filelines.0
       buildme2=buildme2||filelines.oo||crlf
    end
    buildme=buildme||buildme2||crlf

/* before writing, are we out of time? */
    namesl=make_brdlist(broadcastlist.nthforum,autosubscribe.nthforum,theadd0)
    if namesl<>' ' then do
        foo=time('e')+words(namesl)    /* assume 1 second per name */
       if foo>limit_secs then do
           errlist=errlist'<li><p>ReadMail -- Skipping message; no time to process broadcast list: ' broadcastlist.nthforum
           iterate              /* but maybe time to do other messages */
        end           /* note: we retain message in inbox, so it can be processed later */
    end  /*  namesl non empty */
    

/* 5) got time, so  write it out */
   if noarchive=0 then do               /* if noarchive, then do not write to message box */
     lfile=messbox_dir||aforum||'.log'  /* default message box */

     foo=sref_open_read(lfile,30,'BOTH')
     if foo<0 then do
        anerr= "  Warning: ReadMail -- Unable to find message box: " lfile
        errlist=errlist'<li>'anerr
        if verbose>1 then say anerr
        iterate
     end
     dingo=chars(lfile)
     foob=charin(lfile,dingo,1)
     if foob<>'1a'x then dingo=dingo+1
     foo=charout(lfile,buildme,dingo)
     foo=stream(lfile,'c','close')

     if verbose>1 then say " Recording message: " asubject "; to  " aforum " message box "
     ndone.nthforum=ndone.nthforum+1
     ndone.0=ndone.0+1
   end

/* delete or move the NOW-RECORDED message file */
  foo=remove_message(fils.afil)

  broadcast_name=toforum

/* 6) Send verification to theadd0? */
   if  autoverify=1 & smtp_gateway<>' 'then do
       amessage='Your e-mail submission: ' asubject ||crlf
       amessage=amessage||'to the ' || aforum ||' discussion group at '||servername||crlf
       amessage=amessage||' ... was successfully processed on '||date2
       call send_verify('Status of submission ')
    end


/* 7) Rebroadcast (use nameSL determined above) ? */
 
   if namesl=' ' then  iterate          /* no or empty broadcast list */


/* put together a forwarding messabe */
   bsubject='Subject: '||asubject||' (forwarded)'

   dacontents=date3||crlf

      dacontents='Mime-Version: 1.0'||crlf||'Content-Type: text/plain; charset="us-ascii"'||crlf
      dacontents=dacontents||'To:  '||Aforum||' discussion group <>'||crlf
      dacontents=dacontents||'From: Message processor <'||toforum||'>'||crlf

      dacontents=dacontents||bsubject||crlf
 
/* put notes in message body */
      dacontents=dacontents||crlf||'Message sent to '||aforum ' discussion group at '||forum_host||crlf
      dacontents=dacontents||'Date recieved: '||date2||crlf
      dacontents=dacontents||'Sent by: '||fromwho||crlf
      dacontents=dacontents||'Subject: '||asubject||crlf
      dacontents=dacontents||crlf||buildme2
   if verbose>1 then
       say ' Attempting to broadcast message to ' words(namesl) ' in list: ' broadcastlist.nthforum
   status=sref_mailit(namesl,dacontents,smtp_gateway ,'<'||broadcast_name||'>',verbose)
   if status=0 then do
         anerr=" Warning: ReadMail --Broadcasting failed "
         if verbose>1  then say anerr
         errlist=errlist'<li>'anerr
         iterate
   end


end             /* scan all files llop */


/* write status report */
 call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
 call lineout tempfile, "<html><head><title>READMAIL status</title>"
 call lineout tempfile, "</head><body>"
  call lineout tempfile,' <h2> ' ndone.0 ' messages processes by READMAIL  </h2>'
 if ndone.0=0 then do
        call lineout tempfile,' No mail messages were found for currently defined discussion groups '
 end
 else do
    call lineout tempfile,' The following message boxes were updated: '
    call lineout tempfile,'  <ol>'
    do jt=1 to forum.0
         if ndone.jt=0 then iterate
         call lineout tempfile,' <li> ' ndone.jt ' messages written to ' forum.jt
    end
    call lineout tempfile,'  </ol>'
 end  
 if errlist<>' ' then do
      call lineout tempfile,' <p> Notes: <ul>'
      call lineout tempfile,errlist
      call lineout tempfile,'</ul><p>'
 end  /* Do */
/* all done, write final end-of-html elements */
 call lineout tempfile,'<body> </html>'
 call lineout tempfile
 fii=value('SREF_PREFIX',,'os2environment')
 if fii<>'' then 
        foo=sref_gos('FILE ERASE TYPE text/html NAME' tempfile)
 else
      'FILE ERASE TYPE text/html NAME' tempfile

return ' done with readmail '


/* ------------------  check use list  ---------- */
/* check user list for theadd, return 1 if found;  0 if not found or if
a "not this guy" found. "not this guy" is VERY IMPORTANT if you have
wildcards (otherwise, you risk endless recursion whenever an error
message is returned from your smtp gateway!
*/
check_user_list:procedure expose users.
parse arg theadd
   isok=0
   parse var theadd dname '@' ddomain
   ddomain2=translate(ddomain,' ','.')

   do   aad=1 to users.0
      a1=strip(translate(users.aad,' ','000d0a1a09'x))
 
      if abbrev(strip(a1),';')=1 | a1=' ' then iterate

      isnot=1
      if abbrev(a1,'NOT')=1 then do
          isnot=-1
          a1=strip(substr(a1,4))
      end

      if theadd=a1 then       /* exact match */
           return isnot         /* if isnot=-1, DO NOT LET IN */
      
      if a1='*' then do      /*  generic wildcard */
          if isnot=-1 then iterate   /* NOT * is ignored */
          isok=1                /* if no NOT match further down, then accept */
      end  /* Do */

/* try non-generic wildcard */
      parse upper var a1  nname '@' ndomain
      if nname<>'*' & nname<>dname   then  iterate    /* does name  NOT match */

/* see if domain matches */
      ndomain2=translate(ndomain,' ','.')
      if words(ddomain2) <> words(ndomain2) then iterate /* no matching domain */
      do mw=1 to words(ddomain2) 
         if word(ddomain2,mw)=word(ndomain2,mw) | word(ndomain2,mw)='*' then
                iterate
         iterate aad            /* no match */
      end

/* if here, got a matching domain  */
     if isnot=-1 then return isnot    /* forbiddens trump * permissions */

     isok=1             /* but keep looking for possible NOT entries */

   end                     /*aad: search users */
   return isok




/* ************************* */
/* remoove (Delete or archive) a message file */
remove_message:procedure expose mail_archive errlist verbose
parse arg thefil
  IF MAIL_ARCHIVE=0 then do
      FOO=SYSFILEDELETE(thefil)          /* DELETE THE MESSAGE */
      if foo<>0 then do
         anerr=' Warning: READMAIL -- error ' foo ' trying to delete ' thefil
         if verbose>1 then say anerr
         errlist=errlist'<li>'anerr
      end
  end
  ELSE do                       /* MOTE TO ARCHIVE DIRECTORY */
      eeh=mail_archive||filespec('N',thefil)
      FOO=DOSRENAME(theFIL,eeh)
      if foo=0 then do
         anerr=' Warning: READMAIL -- could not archive ' thefil ' to ' eeh ' (delete instead)'
         FOO=SYSFILEDELETE(thefil)          /* DELETE THE MESSAGE */
         if verbose>1 then say anerr
         errlist=errlist'<li>'anerr
      end
  end
  return 1



/* ************************ */
/* get list e-mail addresses to broadcast to */
make_brdlist:procedure expose verbose smtp_gateway errlist
parse arg ylfile,asub,theadd0

 if ylfile=' ' | smtp_gateway=' ' then return ' '

/* else, scarf up the  broadcast_list */
   if stream(ylfile,'c','query exists')=' ' then do
           anerr="ReadMail -- Broadcast denied (no message list found: " ylfile
           if verbose>1 then say anerr
           errlist=errlist'<li>'anerr
           return ' '
   end
   puds=fileread(ylfile,yllist)

   namesl=' '
   do ik=1 to yllist.0               /* grab the names */
         yllist.ik=translate(yllist.ik,' ','000d0a1a09'x)
         if abbrev(strip(yllist.ik),';')=1 | yllist.ik=' ' then iterate  /*comment skips */
         if pos('*',yllist.ik)>0 then do
             if verbose>1  then do
                  anerr='Warning: ReadMail -- Bad entry in broadcast list not used: ' yllist.ik
                  errlist=errlist'<li>'anerr
                  if verbose > 1 then say anerr
                  iterate
             end  /* Do */
          end

/* if autosubscribe=2, then check for "own name", and do not include if
   found */
        if yllist.ik=theadd0 & asub=2 then iterate
         namesl=namesl||' '||yllist.ik
   end

  if namesl=" " then do
           anerr='Warning: ReadMail -- Empty broadcast list '
           if verbose>1 then say anerr
           errlist=errlist'<li>'anerr
           iterate      /* since no more actions for this file */
  end  /* Do */
  return namesl




/*************/
byebye:procedure expose tempfile
parse arg inbox_dir

    call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
    call lineout tempfile, "<html><head><title>READMAIL failed</title>"
    call lineout tempfile, "</head><body>"
    call lineout tempfile,' <h2>Error in READMAIL </h2>'
    call lineout tempfile,' Error reading IN-box directory: ' inbox_dir

/* all done, write final end-of-html elements */
    call lineout tempfile,'<body> </html>'
    call lineout tempfile
    return 0

/*************/
bad_archive:procedure expose tempfile
parse arg mail_archive,inbox_dir

    call lineout tempfile, '<!doctype html public "-//IETF//DTD HTML 2.0//EN">'
    call lineout tempfile, "<html><head><title>READMAIL failed</title>"
    call lineout tempfile, "</head><body>"
    call lineout tempfile,' <h2>Error in READMAIL </h2>'
    call lineout tempfile,' Bad mail archive directory: ' maiL_archive
    call lineout tempfile,' <p>(should be in same drive as In-box directory: ' inbox_dir ')'

/* all done, write final end-of-html elements */
    call lineout tempfile,'<body> </html>'
    call lineout tempfile
    return 0


/*  ----------------------------------------------------- */
send_verify:            /* send verify message. uses globals */
parse arg dasub
      gmtoff=gmtoffset()
      if datatype(gmtoff)<>'NUM' then
            gmtoff=' '
      else
          gmtoff=100*gmtoff/(60*60)
      date3='Date: ' left(date('w'),3)||', '||date('n')||' '||time('n')||' '||gmtoff
      dacontents=date3||crlf

      dacontents='Mime-Version: 1.0'||crlf||'Content-Type: text/plain; charset="us-ascii"'||crlf
      dacontents=dacontents||'To: '||thename||'<'||theadd0||'>'||crlf
      dacontents=dacontents||'From: '||aforum||' Discussion group <_@_>'||crlf

      dacontents=dacontents||'Subject:'||dasub||crlf

      dacontents=dacontents||crlf||amessage

      if verbose>1 then
        say " Attempting to mail verification to "  theadd0
      status=sref_mailit(theadd0,dacontents,smtp_gateway ,'<>',verbose)

      if status=0 then do
          anerr=" Warning: ReadMail -- unable to send verification to: " theadd0
          if verbose>1 then say anerr
          errlist=errlist'<li>'anerr
      end
      return 0


/* -------------------------------------- */
remove_user:procedure expose errlist verbose
parse upper arg theadd0,ufile

crlf = '0d0a'x
aneol=crlf

ause=sref_open_read(ufile,30,'BOTH')
if ause<0 then                  /* couldn't get it */
  return -1
ause=charin(ufile,1,chars(ufile))
ause=translate(ause,' ','001a'x)
filelines.0=0
i=0 ; nfound=0
do until ause=""
  parse var ause aline (aneol) ause
  aline=strip(aline)
  if upper(aline)=upper(theadd0) then do
      nfound=nfound+1
      iterate
  end  /* Do */
  i=i+1
  filelines.i=aline
end
filelines.0=i
a=stream(ufile,'c','close')
foo=filewrite(ufile,filelines,'R',i,1)
return nfound

